@@ -63,9 +63,6 @@ type Application struct {
 	// VeryVerbose enables a higher level of verbosity in logging output.
 	VeryVerbose bool `flag:"vv,veryverbose" help:"very verbose output"`
 
-	// Control ocagent export of telemetry
-	OCAgent string `flag:"ocagent" help:"the address of the ocagent (e.g. http://localhost:55678), or off"`
-
 	// PrepareOptions is called to update the options when a new view is built.
 	// It is primarily to allow the behavior of gopls to be modified by hooks.
 	PrepareOptions func(*settings.Options)
@@ -98,8 +95,6 @@ func (app *Application) verbose() bool {
 // New returns a new Application ready to run.
 func New() *Application {
 	app := &Application{
-		OCAgent: "off", //TODO: Remove this line to default the exporter to on
-
 		Serve: Serve{
 			RemoteListenTimeout: 1 * time.Minute,
 		},
@@ -220,7 +215,7 @@ func isZeroValue(f *flag.Flag, value string) bool {
 	// This works unless the Value type is itself an interface type.
 	typ := reflect.TypeOf(f.Value)
 	var z reflect.Value
-	if typ.Kind() == reflect.Ptr {
+	if typ.Kind() == reflect.Pointer {
 		z = reflect.New(typ.Elem())
 	} else {
 		z = reflect.Zero(typ)
@@ -238,7 +233,7 @@ func (app *Application) Run(ctx context.Context, args ...string) error {
 	// executable, and immediately runs a gc.
 	filecache.Start()
 
-	ctx = debug.WithInstance(ctx, app.OCAgent)
+	ctx = debug.WithInstance(ctx)
 	if len(args) == 0 {
 		s := flag.NewFlagSet(app.Name(), flag.ExitOnError)
 		return tool.Run(ctx, s, &app.Serve, args)
@@ -284,7 +279,7 @@ func (app *Application) internalCommands() []tool.Application {
 func (app *Application) featureCommands() []tool.Application {
 	return []tool.Application{
 		&callHierarchy{app: app},
-		&check{app: app},
+		&check{app: app, Severity: "warning"},
 		&codeaction{app: app},
 		&codelens{app: app},
 		&definition{app: app},
@@ -310,11 +305,6 @@ func (app *Application) featureCommands() []tool.Application {
 	}
 }
 
-var (
-	internalMu          sync.Mutex
-	internalConnections = make(map[string]*connection)
-)
-
 // connect creates and initializes a new in-process gopls session.
 func (app *Application) connect(ctx context.Context) (*connection, error) {
 	client := newClient(app)
@@ -353,7 +343,8 @@ func (c *connection) initialize(ctx context.Context, options func(*settings.Opti
 
 	// Make sure to respect configured options when sending initialize request.
 	opts := settings.DefaultOptions(options)
-	// If you add an additional option here, you must update the map key in connect.
+	// If you add an additional option here,
+	// you must update the map key of settings.DefaultOptions called in (*Application).connect.
 	params.Capabilities.TextDocument.Hover = &protocol.HoverClientCapabilities{
 		ContentFormat: []protocol.MarkupKind{opts.PreferredContentFormat},
 	}
@@ -361,7 +352,7 @@ func (c *connection) initialize(ctx context.Context, options func(*settings.Opti
 	params.Capabilities.TextDocument.SemanticTokens = protocol.SemanticTokensClientCapabilities{}
 	params.Capabilities.TextDocument.SemanticTokens.Formats = []protocol.TokenFormat{"relative"}
 	params.Capabilities.TextDocument.SemanticTokens.Requests.Range = &protocol.Or_ClientSemanticTokensRequestOptions_range{Value: true}
-	//params.Capabilities.TextDocument.SemanticTokens.Requests.Range.Value = true
+	// params.Capabilities.TextDocument.SemanticTokens.Requests.Range.Value = true
 	params.Capabilities.TextDocument.SemanticTokens.Requests.Full = &protocol.Or_ClientSemanticTokensRequestOptions_full{Value: true}
 	params.Capabilities.TextDocument.SemanticTokens.TokenTypes = moreslices.ConvertStrings[string](semtok.TokenTypes)
 	params.Capabilities.TextDocument.SemanticTokens.TokenModifiers = moreslices.ConvertStrings[string](semtok.TokenModifiers)
@@ -373,14 +364,17 @@ func (c *connection) initialize(ctx context.Context, options func(*settings.Opti
 		},
 	}
 	params.Capabilities.Window.WorkDoneProgress = true
+	params.Capabilities.Workspace.FileOperations = &protocol.FileOperationClientCapabilities{
+		DidCreate: true,
+	}
 
-	params.InitializationOptions = map[string]interface{}{
+	params.InitializationOptions = map[string]any{
 		"symbolMatcher": string(opts.SymbolMatcher),
 	}
-	if c.initializeResult, err = c.Server.Initialize(ctx, params); err != nil {
+	if c.initializeResult, err = c.Initialize(ctx, params); err != nil {
 		return err
 	}
-	if err := c.Server.Initialized(ctx, &protocol.InitializedParams{}); err != nil {
+	if err := c.Initialized(ctx, &protocol.InitializedParams{}); err != nil {
 		return err
 	}
 	return nil
@@ -473,7 +467,7 @@ func (c *cmdClient) LogMessage(ctx context.Context, p *protocol.LogMessageParams
 	return nil
 }
 
-func (c *cmdClient) Event(ctx context.Context, t *interface{}) error { return nil }
+func (c *cmdClient) Event(ctx context.Context, t *any) error { return nil }
 
 func (c *cmdClient) RegisterCapability(ctx context.Context, p *protocol.RegistrationParams) error {
 	return nil
@@ -487,13 +481,13 @@ func (c *cmdClient) WorkspaceFolders(ctx context.Context) ([]protocol.WorkspaceF
 	return nil, nil
 }
 
-func (c *cmdClient) Configuration(ctx context.Context, p *protocol.ParamConfiguration) ([]interface{}, error) {
-	results := make([]interface{}, len(p.Items))
+func (c *cmdClient) Configuration(ctx context.Context, p *protocol.ParamConfiguration) ([]any, error) {
+	results := make([]any, len(p.Items))
 	for i, item := range p.Items {
 		if item.Section != "gopls" {
 			continue
 		}
-		m := map[string]interface{}{
+		m := map[string]any{
 			"analyses": map[string]any{
 				"fillreturns":    true,
 				"nonewvars":      true,
@@ -647,6 +641,10 @@ func updateFile(filename string, old, new []byte, edits []diff.Edit, flags *Edit
 
 func (c *cmdClient) PublishDiagnostics(ctx context.Context, p *protocol.PublishDiagnosticsParams) error {
 	// Don't worry about diagnostics without versions.
+	//
+	// (Note: the representation of PublishDiagnosticsParams
+	// cannot distinguish a missing Version from v0, but the
+	// server never sends back an explicit zero.)
 	if p.Version == 0 {
 		return nil
 	}
@@ -663,7 +661,7 @@ func (c *cmdClient) PublishDiagnostics(ctx context.Context, p *protocol.PublishD
 	// TODO(golang/go#60122): replace the gopls.diagnose_files
 	// command with support for textDocument/diagnostic,
 	// so that we don't need to do this de-duplication.
-	type key [6]interface{}
+	type key [6]any
 	seen := make(map[key]bool)
 	out := file.diagnostics[:0]
 	for _, d := range file.diagnostics {
@@ -778,10 +776,25 @@ func (c *connection) openFile(ctx context.Context, uri protocol.DocumentURI) (*c
 		return nil, file.err
 	}
 
+	// Choose language ID from file extension.
+	var langID protocol.LanguageKind // "" eventually maps to file.UnknownKind
+	switch filepath.Ext(uri.Path()) {
+	case ".go":
+		langID = "go"
+	case ".mod":
+		langID = "go.mod"
+	case ".sum":
+		langID = "go.sum"
+	case ".work":
+		langID = "go.work"
+	case ".s":
+		langID = "go.s"
+	}
+
 	p := &protocol.DidOpenTextDocumentParams{
 		TextDocument: protocol.TextDocumentItem{
 			URI:        uri,
-			LanguageID: "go",
+			LanguageID: langID,
 			Version:    1,
 			Text:       string(file.mapper.Content),
 		},
@@ -812,10 +825,10 @@ func (c *connection) diagnoseFiles(ctx context.Context, files []protocol.Documen
 }
 
 func (c *connection) terminate(ctx context.Context) {
-	//TODO: do we need to handle errors on these calls?
-	c.Shutdown(ctx)
-	//TODO: right now calling exit terminates the process, we should rethink that
-	//server.Exit(ctx)
+	// TODO: do we need to handle errors on these calls?
+	c.Shutdown(ctx) // ignore error
+	// TODO: right now calling exit terminates the process, we should rethink that
+	// server.Exit(ctx)
 }
 
 // Implement io.Closer.
@@ -878,7 +891,7 @@ func (f *cmdFile) spanLocation(s span) (protocol.Location, error) {
 	if err != nil {
 		return protocol.Location{}, err
 	}
-	return f.mapper.RangeLocation(rng), nil
+	return f.mapper.URI.Location(rng), nil
 }
 
 // spanRange converts a (UTF-8) span to a protocol (UTF-16) range.
@@ -890,7 +903,7 @@ func (f *cmdFile) spanRange(s span) (protocol.Range, error) {
 	// case-sensitive directories. The authoritative answer
 	// requires querying the file system, and we don't want
 	// to do that.
-	if !strings.EqualFold(filepath.Base(string(f.mapper.URI)), filepath.Base(string(s.URI()))) {
+	if !strings.EqualFold(f.mapper.URI.Base(), s.URI().Base()) {
 		return protocol.Range{}, bugpkg.Errorf("mapper is for file %q instead of %q", f.mapper.URI, s.URI())
 	}
 	start, err := pointPosition(f.mapper, s.Start())
