@@ -108,6 +108,12 @@ var C int
 -- c/c2.go --
 package c
 var C int
+-- d/d.go --
+package d
+
+import "io/ioutil"
+
+var _ = ioutil.ReadFile
 `)
 
 	// no files
@@ -141,6 +147,22 @@ var C int
 		res.checkStdout(`c2.go:2:5-6: C redeclared in this block`)
 		res.checkStdout(`c.go:2:5-6: - other declaration of C`)
 	}
+
+	// No deprecated (hint) diagnostic without -severity.
+	{
+		res := gopls(t, tree, "check", "./d/d.go")
+		res.checkExit(true)
+		if len(res.stdout) > 0 {
+			t.Errorf("check ./d/d.go returned unexpected output:\n%s", res.stdout)
+		}
+	}
+
+	// Deprecated (hint) diagnostics with -severity=hint
+	{
+		res := gopls(t, tree, "check", "-severity=hint", "./d/d.go")
+		res.checkExit(true)
+		res.checkStdout(`ioutil.ReadFile is deprecated`)
+	}
 }
 
 // TestCallHierarchy tests the 'call_hierarchy' subcommand (call_hierarchy.go).
@@ -486,6 +508,14 @@ func f() {
 func TestImplementations(t *testing.T) {
 	t.Parallel()
 
+	// types.CheckExpr, now used in the rangeint modernizer, had a
+	// data race (#71817) that was fixed in go1.25 and backported
+	// to go1.24 but not to go1.23. Although in principle it could
+	// affect a lot of tests, it (weirdly) only seems to show up
+	// in this one (#72082). Rather than backport again, we
+	// suppress this test.
+	testenv.NeedsGo1Point(t, 24)
+
 	tree := writeTree(t, `
 -- a.go --
 package a
@@ -930,7 +960,7 @@ package foo
 		res3 := goplsWithEnv(t, tree, []string{"GOPACKAGESDRIVER=off"}, "stats", "-anon")
 		res3.checkExit(true)
 
-		var statsAsMap3 map[string]interface{}
+		var statsAsMap3 map[string]any
 		if err := json.Unmarshal([]byte(res3.stdout), &statsAsMap3); err != nil {
 			t.Fatalf("failed to unmarshal JSON output of stats command: %v", err)
 		}
@@ -980,9 +1010,9 @@ type C struct{}
 		res := gopls(t, tree, "codeaction", "-title=Browse.*doc", "a/a.go")
 		res.checkExit(true)
 		got := res.stdout
-		want := `command	"Browse gopls feature documentation" [gopls.doc.features]` +
+		want := `command	"Browse documentation for package a" [source.doc]` +
 			"\n" +
-			`command	"Browse documentation for package a" [source.doc]` +
+			`command	"Browse gopls feature documentation" [gopls.doc.features]` +
 			"\n"
 		if got != want {
 			t.Errorf("codeaction: got <<%s>>, want <<%s>>\nstderr:\n%s", got, want, res.stderr)
@@ -1212,7 +1242,7 @@ func (res *result) checkOutput(pattern, name, content string) {
 }
 
 // toJSON decodes res.stdout as JSON into to *ptr and reports its success.
-func (res *result) toJSON(ptr interface{}) bool {
+func (res *result) toJSON(ptr any) bool {
 	if err := json.Unmarshal([]byte(res.stdout), ptr); err != nil {
 		res.t.Errorf("invalid JSON %v", err)
 		return false
